local Ch = CBW_Chaos
local Lib = CBW_Chaos_Library
local m = mobjinfo

freeslot('mt_chaosturretcapsule',
	'S_CHAOSTURRETCAPSULE',
	'S_CHAOSTURRETCAPSULE_pain',
	'S_CHAOSTURRETCAPSULE_die1',
	'S_CHAOSTURRETCAPSULE_die2',
	'S_CHAOSTURRETCAPSULE_die3',
	'S_CHAOSTURRETCAPSULE_SHOOT',
	"S_CHAOSTURRETCAPSULE_SHOOT2",
	'SPR_CPTR')

mobjinfo[MT_CHAOSTURRETCAPSULE] = {
	spawnstate		= S_CHAOSTURRETCAPSULE, 
	painstate		= S_CHAOSTURRETCAPSULE_PAIN,
	deathstate		= S_CHAOSTURRETCAPSULE_DIE1,
	seestate		= S_CHAOSTURRETCAPSULE_SHOOT,
	height			= 120*FRACUNIT,
	radius			= 32*FRACUNIT,
	spawnhealth		= 3,
	damage			= 1,
	painsound		= sfx_dmpain,
	deathsound		= sfx_s3kb4,
	flags			= MF_SPECIAL|MF_BOSS|MF_SHOOTABLE,
	speed			= FRACUNIT,
}

local info = m[MT_CHAOSTURRETCAPSULE]

info.name = 'Eggplant TurretCapsule (Laser)'
info.article = 'a'
info.chaosmode = true
info.difficulty = 2
info.spawntype = 'target'
info.spawnsubtype = {'sniper','land'}
info.spawnpriority = 0
info.spawnbiome = nil
info.spawnweight = 2
info.spawnfrequency = 2


info.bossregulate	= 1
info.flashingtime	= PAIN_SHORT
info.grounded = true
info.harmless = true
info.bumptype = BUMP_HEAVY
info.color 	= GETCOLOR_BIOME

states[S_CHAOSTURRETCAPSULE] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_Look,
	tics		= 1,
	nextstate	= S_CHAOSTURRETCAPSULE,
}

states[S_CHAOSTURRETCAPSULE_SHOOT] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_FaceTarget,
	tics		= 1,
	nextstate	= S_CHAOSTURRETCAPSULE_SHOOT2,
}

states[S_CHAOSTURRETCAPSULE_SHOOT2] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_SuperFireShot,
	var1		= MT_ROCKET,
	var2		= (92),
	tics		= 25,
	nextstate	= S_CHAOSTURRETCAPSULE,
}

states[S_CHAOSTURRETCAPSULE_PAIN] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_Pain,
	tics		= PAIN_SHORT,
	nextstate	= S_CHAOSTURRETCAPSULE,
}

states[S_CHAOSTURRETCAPSULE_DIE1] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_BossScream,
	tics		= 2,
	nextstate	= S_CHAOSTURRETCAPSULE_DIE2,
}

states[S_CHAOSTURRETCAPSULE_DIE2] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_BossScream,
	tics		= 2,
	nextstate	= S_CHAOSTURRETCAPSULE_DIE3,
}
states[S_CHAOSTURRETCAPSULE_DIE3] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_Repeat,
	tics		= 2,
	var1		= 7,
	var2		= S_CHAOSTURRETCAPSULE_DIE1,
	nextstate	= S_BUMBLEBORE_DIE
}

addHook('MobjSpawn',function(mo)
	Lib.EnemyInit(mo)
	//Do bumper spawns
	mo.spawntype = MT_BUMPER
	if Ch.WaveNum > CHAOS_BOSSWAVE or Lib.CV_Difficulty.value == 5
		mo.spawncount = min(2, Lib.CV_Difficulty.value)
		Lib.EnemySpawnChildren(mo)
	end
end,MT_CHAOSTURRETCAPSULE)

addHook('MobjThinker',function(mo)
	if not(mo.health) return end
	local maxdist = mo.scale * 128
	local speed = FixedMul(m[mo.type].speed, mo.scale)
	mo.angle = $ + FixedAngle(speed)
	Lib.EnemyTic_FamilyThink(mo)
	for n,obj in pairs(mo.children)
		obj.flags = $|MF_NOCLIP|MF_NOCLIPHEIGHT
		local dist = min(maxdist, R_PointToDist2(mo.x, mo.y, obj.x, obj.y) + speed)
		local ang1 = R_PointToAngle2(mo.x, mo.y, obj.x, obj.y)
		local ang2 = mo.angle + FixedAngle(FRACUNIT / #mo.children * 360 * n)
		local diff = ang2-ang1
		local angle = ang1 + diff/16
		local x = mo.x + P_ReturnThrustX(nil,angle,dist)
		local y = mo.y + P_ReturnThrustY(nil,angle,dist)
		
		local height = (obj.z-mo.z)
		local z = mo.z + height + (24*mo.scale - height) / 16
		
		P_TeleportMove(obj, x, y, z)
	end
	
end,MT_CHAOSTURRETCAPSULE)

addHook('MobjDeath',function(mo)
	Lib.EnemyKillFamily(mo)
end,MT_CHAOSTURRETCAPSULE)

addHook('MobjRemoved',function(mo)
	Lib.EnemyKillFamily(mo)
	Lib.SpawnFlowers(mo,4,1,true)
	Lib.SpawnFlowers(mo,8,2)
	Lib.SpawnFlowers(mo,14,3)
end,MT_CHAOSTURRETCAPSULE)


freeslot('mt_CHAOSTURRETCAPSULEBOUNCE',
	'S_CHAOSTURRETCAPSULEBOUNCE',
	'S_CHAOSTURRETCAPSULEBOUNCE_pain',
	'S_CHAOSTURRETCAPSULEBOUNCE_die1',
	'S_CHAOSTURRETCAPSULEBOUNCE_die2',
	'S_CHAOSTURRETCAPSULEBOUNCE_die3',
	'S_CHAOSTURRETCAPSULEBOUNCE_SHOOT',
	"S_CHAOSTURRETCAPSULEBOUNCE_SHOOT2")

mobjinfo[MT_CHAOSTURRETCAPSULEBOUNCE] = {
	spawnstate		= S_CHAOSTURRETCAPSULEBOUNCE, 
	painstate		= S_CHAOSTURRETCAPSULEBOUNCE_PAIN,
	deathstate		= S_CHAOSTURRETCAPSULEBOUNCE_DIE1,
	seestate		= S_CHAOSTURRETCAPSULEBOUNCE_SHOOT,
	height			= 120*FRACUNIT,
	radius			= 32*FRACUNIT,
	spawnhealth		= 3,
	damage			= 1,
	painsound		= sfx_dmpain,
	deathsound		= sfx_s3kb4,
	flags			= MF_SPECIAL|MF_BOSS|MF_SHOOTABLE,
	speed			= FRACUNIT,
}

local info = m[MT_CHAOSTURRETCAPSULEBOUNCE]

info.name = 'Eggplant TurretCapsule (Rocket)'
info.article = 'a'
info.chaosmode = true
info.difficulty = 2
info.spawntype = 'target'
info.spawnsubtype = {'sniper','land'}
info.spawnpriority = 0
info.spawnbiome = nil
info.spawnweight = 3
info.spawnfrequency = 1


info.bossregulate	= 1
info.flashingtime	= PAIN_LONG
info.grounded = true
info.harmless = true
info.bumptype = BUMP_HEAVY
info.color 	= GETCOLOR_BIOME

states[S_CHAOSTURRETCAPSULEBOUNCE] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_Look,
	tics		= 1,
	nextstate	= S_CHAOSTURRETCAPSULEBOUNCE,
}

states[S_CHAOSTURRETCAPSULEBOUNCE_SHOOT] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_FaceTarget,
	tics		= 1,
	nextstate	= S_CHAOSTURRETCAPSULEBOUNCE_SHOOT2,
}

states[S_CHAOSTURRETCAPSULEBOUNCE_SHOOT2] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_SuperFireShot,
	var1		= MT_CYBRAKDEMON_MISSILE,
	var2		= (92),
	tics		= 175,
	nextstate	= S_CHAOSTURRETCAPSULEBOUNCE,
}

states[S_CHAOSTURRETCAPSULEBOUNCE_PAIN] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_Pain,
	tics		= PAIN_LONG,
	nextstate	= S_CHAOSTURRETCAPSULEBOUNCE,
}

states[S_CHAOSTURRETCAPSULEBOUNCE_DIE1] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_BossScream,
	tics		= 2,
	nextstate	= S_CHAOSTURRETCAPSULEBOUNCE_DIE2,
}

states[S_CHAOSTURRETCAPSULEBOUNCE_DIE2] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_BossScream,
	tics		= 2,
	nextstate	= S_CHAOSTURRETCAPSULEBOUNCE_DIE3,
}
states[S_CHAOSTURRETCAPSULEBOUNCE_DIE3] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_Repeat,
	tics		= 2,
	var1		= 7,
	var2		= S_CHAOSTURRETCAPSULEBOUNCE_DIE1,
	nextstate	= S_BUMBLEBORE_DIE
}

addHook('MobjSpawn',function(mo)
	Lib.EnemyInit(mo)
	//Do bumper spawns
	mo.spawntype = MT_BUMPER
	if Ch.WaveNum > CHAOS_BOSSWAVE or Lib.CV_Difficulty.value == 5
		mo.spawncount = min(4, Lib.CV_Difficulty.value)
		Lib.EnemySpawnChildren(mo)
	end
end,MT_CHAOSTURRETCAPSULEBOUNCE)

addHook('MobjThinker',function(mo)
	if not(mo.health) return end
	local maxdist = mo.scale * 128
	local speed = FixedMul(m[mo.type].speed, mo.scale)
	mo.angle = $ + FixedAngle(speed)
	Lib.EnemyTic_FamilyThink(mo)
	for n,obj in pairs(mo.children)
		obj.flags = $|MF_NOCLIP|MF_NOCLIPHEIGHT
		local dist = min(maxdist, R_PointToDist2(mo.x, mo.y, obj.x, obj.y) + speed)
		local ang1 = R_PointToAngle2(mo.x, mo.y, obj.x, obj.y)
		local ang2 = mo.angle + FixedAngle(FRACUNIT / #mo.children * 360 * n)
		local diff = ang2-ang1
		local angle = ang1 + diff/16
		local x = mo.x + P_ReturnThrustX(nil,angle,dist)
		local y = mo.y + P_ReturnThrustY(nil,angle,dist)
		
		local height = (obj.z-mo.z)
		local z = mo.z + height + (24*mo.scale - height) / 16
		
		P_TeleportMove(obj, x, y, z)
	end
	
end,MT_CHAOSTURRETCAPSULEBOUNCE)

addHook('MobjDeath',function(mo)
	Lib.EnemyKillFamily(mo)
end,MT_CHAOSTURRETCAPSULEBOUNCE)

addHook('MobjRemoved',function(mo)
	Lib.EnemyKillFamily(mo)
	Lib.SpawnFlowers(mo,4,1,true)
	Lib.SpawnFlowers(mo,8,2)
	Lib.SpawnFlowers(mo,14,3)
end,MT_CHAOSTURRETCAPSULEBOUNCE)

freeslot('mt_CHAOSTURRETCAPSULESCATTER',
	'S_CHAOSTURRETCAPSULESCATTER',
	'S_CHAOSTURRETCAPSULESCATTER_pain',
	'S_CHAOSTURRETCAPSULESCATTER_die1',
	'S_CHAOSTURRETCAPSULESCATTER_die2',
	'S_CHAOSTURRETCAPSULESCATTER_die3',
	'S_CHAOSTURRETCAPSULESCATTER_SHOOT',
	"S_CHAOSTURRETCAPSULESCATTER_SHOOT2")

mobjinfo[MT_CHAOSTURRETCAPSULESCATTER] = {
	spawnstate		= S_CHAOSTURRETCAPSULESCATTER, 
	painstate		= S_CHAOSTURRETCAPSULESCATTER_PAIN,
	deathstate		= S_CHAOSTURRETCAPSULESCATTER_DIE1,
	seestate		= S_CHAOSTURRETCAPSULESCATTER_SHOOT,
	height			= 120*FRACUNIT,
	radius			= 32*FRACUNIT,
	spawnhealth		= 4,
	damage			= 1,
	painsound		= sfx_dmpain,
	deathsound		= sfx_s3kb4,
	flags			= MF_SPECIAL|MF_BOSS|MF_SHOOTABLE,
	speed			= FRACUNIT,
}

local info = m[MT_CHAOSTURRETCAPSULESCATTER]

info.name = 'Eggplant TurretCapsule (ScatterRing)'
info.article = 'a'
info.chaosmode = true
info.difficulty = 1
info.spawntype = 'target'
info.spawnsubtype = {'sniper','land'}
info.spawnpriority = 0
info.spawnbiome = nil
info.spawnweight = 3
info.spawnfrequency = 2


info.bossregulate	= 1
info.flashingtime	= PAIN_SHORT
info.grounded = true
info.harmless = true
info.bumptype = BUMP_HEAVY
info.color 	= GETCOLOR_BIOME

states[S_CHAOSTURRETCAPSULESCATTER] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_Look,
	tics		= 1,
	nextstate	= S_CHAOSTURRETCAPSULESCATTER,
}

states[S_CHAOSTURRETCAPSULESCATTER_SHOOT] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_FaceTarget,
	tics		= 1,
	nextstate	= S_CHAOSTURRETCAPSULESCATTER_SHOOT2,
}

states[S_CHAOSTURRETCAPSULESCATTER_SHOOT2] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_SuperFireShot,
	var1		= MT_THROWNSCATTER,
	var2		= (92),
	tics		= 70,
	nextstate	= S_CHAOSTURRETCAPSULESCATTER,
}

states[S_CHAOSTURRETCAPSULESCATTER_PAIN] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_Pain,
	tics		= PAIN_SHORT,
	nextstate	= S_CHAOSTURRETCAPSULESCATTER,
}

states[S_CHAOSTURRETCAPSULESCATTER_DIE1] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_BossScream,
	tics		= 2,
	nextstate	= S_CHAOSTURRETCAPSULESCATTER_DIE2,
}

states[S_CHAOSTURRETCAPSULESCATTER_DIE2] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_BossScream,
	tics		= 2,
	nextstate	= S_CHAOSTURRETCAPSULESCATTER_DIE3,
}
states[S_CHAOSTURRETCAPSULESCATTER_DIE3] = {
	sprite		= SPR_CPTR,
	frame		= A,
	action		= A_Repeat,
	tics		= 2,
	var1		= 7,
	var2		= S_CHAOSTURRETCAPSULESCATTER_DIE1,
	nextstate	= S_BUMBLEBORE_DIE
}

addHook('MobjSpawn',function(mo)
	Lib.EnemyInit(mo)
	//Do bumper spawns
	mo.spawntype = MT_BUMPER
	if Ch.WaveNum > CHAOS_BOSSWAVE or Lib.CV_Difficulty.value == 5
		mo.spawncount = min(2, Lib.CV_Difficulty.value)
		Lib.EnemySpawnChildren(mo)
	end
end,MT_CHAOSTURRETCAPSULESCATTER)

addHook('MobjThinker',function(mo)
	if not(mo.health) return end
	local maxdist = mo.scale * 128
	local speed = FixedMul(m[mo.type].speed, mo.scale)
	mo.angle = $ + FixedAngle(speed)
	Lib.EnemyTic_FamilyThink(mo)
	for n,obj in pairs(mo.children)
		obj.flags = $|MF_NOCLIP|MF_NOCLIPHEIGHT
		local dist = min(maxdist, R_PointToDist2(mo.x, mo.y, obj.x, obj.y) + speed)
		local ang1 = R_PointToAngle2(mo.x, mo.y, obj.x, obj.y)
		local ang2 = mo.angle + FixedAngle(FRACUNIT / #mo.children * 360 * n)
		local diff = ang2-ang1
		local angle = ang1 + diff/16
		local x = mo.x + P_ReturnThrustX(nil,angle,dist)
		local y = mo.y + P_ReturnThrustY(nil,angle,dist)
		
		local height = (obj.z-mo.z)
		local z = mo.z + height + (24*mo.scale - height) / 16
		
		P_TeleportMove(obj, x, y, z)
	end
	
end,MT_CHAOSTURRETCAPSULESCATTER)

addHook('MobjDeath',function(mo)
	Lib.EnemyKillFamily(mo)
end,MT_CHAOSTURRETCAPSULESCATTER)

addHook('MobjRemoved',function(mo)
	Lib.EnemyKillFamily(mo)
	Lib.SpawnFlowers(mo,4,1,true)
	Lib.SpawnFlowers(mo,8,2)
	Lib.SpawnFlowers(mo,14,3)
end,MT_CHAOSTURRETCAPSULESCATTER)
